﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using XNACS1Lib;

namespace ParticleStuff
{
    /// <summary>
    /// Emits a bolt of Lightning
    /// </summary>
    public class LightningEmitter : XNACS1ParticleEmitter
    {
        Vector2 m_CurrentPos;
        float m_ParticleSpacing;

        /// <summary>
        /// constructor for ligtning emitter
        /// </summary>
        /// <param name="center"></param>
        /// <param name="initialParticleLife"></param>
        /// <param name="particleSize"></param>
        /// <param name="particleSpacing">spaceing between the particles</param>
        /// <param name="tex"></param>
        /// <param name="defaultColor"></param>
        public LightningEmitter(Vector2 center, int initialParticleLife, float particleSize, float particleSpacing, string tex, Color defaultColor)
            : base(center, initialParticleLife, particleSize, tex, defaultColor)
        {
            AutoEmitEnabled = false;
            m_CurrentPos = center;
            m_ParticleSpacing = particleSpacing;
        }

        /// <summary>
        /// Shoots a lightning bolt towards the end position
        /// </summary>
        /// <param name="endPos">the target of the bolt</param>
        /// <param name="maxBranches">the max number of branches</param>
        /// <param name="maxBranchSegments">the max number of branch segments</param>
        public void Bolt(Vector2 endPos, int maxBranches, int maxBranchSegments)
        {
            BoltHelper(endPos, maxBranches, Center, -1, maxBranchSegments);
        }

        /// <summary>
        /// a helper method to allows for multiple branching
        /// </summary>
        /// <param name="endPos">target of the bolt</param>
        /// <param name="maxBranches">max number of branches</param>
        /// <param name="startPos">start position of the branch</param>
        /// <param name="maxSegments">max segments</param>
        /// <param name="maxBranchSegments">max segments of branches</param>
        private void BoltHelper(Vector2 endPos, int maxBranches, Vector2 startPos, int maxSegments, int maxBranchSegments)
        {
            Vector2 curPos;
            curPos = startPos;
            float maxAngle = .9f * (float)Math.PI;
            int counter = XNACS1Base.RandomInt(5, 15);
            Vector2 currentDirection = -curPos + endPos;

            int maxBranchCounter = (int)Math.Round((currentDirection.Length() / m_ParticleSpacing) / maxBranches);
            int branchCounter = maxBranchCounter + XNACS1Base.RandomInt(-maxBranchCounter / 3, maxBranchCounter / 3);

            if (currentDirection.Length() < 15 * m_ParticleSpacing)
            {
                currentDirection.Normalize();
                currentDirection *= m_ParticleSpacing;
            }
            else
            {
                currentDirection = Rotate(currentDirection, XNACS1Base.RandomFloat(-maxAngle / 2, maxAngle / 2));
                currentDirection = Vector2.Normalize(currentDirection) * m_ParticleSpacing;
            }


            while (maxSegments != 0 && ((endPos.X != 0 && Math.Abs(curPos.X / endPos.X - 1) > .02)
                || (endPos.Y != 0 && Math.Abs(curPos.Y / endPos.Y - 1) > .02)) &&
                Math.Abs(Vector2.Distance(endPos, curPos)) > m_ParticleSpacing)
            {
                m_CurrentPos = curPos;

                Emit();
                Emit();
                Emit();

                curPos += currentDirection;

                if (branchCounter == 0)
                {
                    branchCounter = maxBranchCounter + XNACS1Base.RandomInt(-maxBranchCounter / 3, maxBranchCounter / 3);
                    if (maxBranches > 1)
                        BoltHelper(endPos, 1, curPos, maxBranchSegments, maxBranchSegments);
                    else
                        BoltHelper(endPos, 0, curPos, maxBranchSegments, maxBranchSegments);
                }
                else
                {
                    branchCounter--;
                }
                if (counter == 0)
                {
                    maxSegments--;
                    counter = XNACS1Base.RandomInt(5, 15);
                    maxAngle *= .80f;
                    currentDirection = -curPos + endPos;
                    if (currentDirection.Length() < 15 * m_ParticleSpacing)
                    {
                        currentDirection.Normalize();
                        currentDirection *= m_ParticleSpacing;
                    }
                    else
                    {
                        currentDirection = Rotate(currentDirection, XNACS1Base.RandomFloat(-maxAngle / 2, maxAngle / 2));
                        currentDirection = Vector2.Normalize(currentDirection) * m_ParticleSpacing;
                    }
                }
                else
                    counter--;
            }
        }

        /// <summary>
        /// rotates the vector by the angle
        /// </summary>
        /// <param name="p">the vector to rotate</param>
        /// <param name="rad">the angle to rotate by</param>
        /// <returns>returns a new vector that has been rotated</returns>
        private Vector2 Rotate(Vector2 p, float rad)
        {
            Vector2 o = Vector2.Zero;

            Vector2 r = new Vector2();
            r.X = o.X + (float)Math.Cos(rad) * (p.X - o.X) - (float)Math.Sin(rad) * (p.Y - o.Y);
            r.Y = o.Y + (float)Math.Sin(rad) * (p.X - o.X) + (float)Math.Cos(rad) * (p.Y - o.Y);
            return r;
        }

        /// <summary>
        /// creates a new particle at the current position
        /// </summary>
        /// <param name="newParticle"></param>
        protected override void NewParticle(XNACS1Particle newParticle)
        {
            newParticle.Center = m_CurrentPos;
        }

        /// <summary>
        /// Overloaded UpdateParticle method, makes particles fade out according to what
        /// percentage of the default life they have remaining.
        /// </summary>
        /// <param name="particle"></param>
        protected override void UpdateParticle(XNACS1Particle particle)
        {
            int temp = (255 * particle.Life) / InitialLife;
            if (temp > 255) temp = 255;
            else if (temp < 0) temp = 0;
            Color tempColor = particle.TextureTintColor;
            tempColor.A = (byte)temp;
            particle.TextureTintColor = tempColor;
        }
    }
}
