﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using XNACS1Lib;

namespace ParticleStuff
{
    /// <summary>
    /// Emits a vortex
    /// </summary>
    public class VortexEmitter : XNACS1ParticleEmitter
    {
        public enum VortexFadeMode
        {
            FadeIn, FadeOut, FadeInOut
        }

        float m_Radius;
        float m_Speed;
        ParticleTravelMode m_Mode;
        float m_DispAmp;
        float m_DispFreq;
        VortexFadeMode m_FadeMode;
        bool m_TravelDirection;

        /// <summary>
        /// Constructor for the Vortex Emitter
        /// </summary>
        /// <param name="center"></param>
        /// <param name="initialParticleLife"></param>
        /// <param name="particleSize"></param>
        /// <param name="tex"></param>
        /// <param name="defaultColor"></param>
        /// <param name="radius">radius of the vortex</param>
        /// <param name="travelMode">travel mode</param>
        /// <param name="travelModeAmplitude">amplitude for the travel mode</param>
        /// <param name="travelModeFrequency">frequency for the travel mode</param>
        /// <param name="fadeMode">fade mode for the vortex</param>
        /// <param name="travelDirection">true for in, false for out</param>
        public VortexEmitter(Vector2 center, int initialParticleLife, float particleSize, string tex, Color defaultColor, float radius,
            ParticleTravelMode travelMode, float travelModeAmplitude, float travelModeFrequency, VortexFadeMode fadeMode, bool travelDirection)
            : base(center, initialParticleLife, particleSize, tex, defaultColor)
        {
            AutoEmitEnabled = true;
            m_Radius = radius;
            m_Speed = radius / initialParticleLife;
            m_Mode = travelMode;
            m_DispAmp = travelModeAmplitude;
            m_DispFreq = travelModeFrequency;
            m_FadeMode = fadeMode;
            m_TravelDirection = travelDirection;
        }

        #region Accessors
        /// <summary>
        /// Sets the travel Direction for new particles
        /// </summary>
        public bool TravelDirection
        {
            get { return m_TravelDirection; }
            set { m_TravelDirection = value; }
        }

        /// <summary>
        /// Sets the fade mode for new particles
        /// </summary>
        public VortexFadeMode FadeMode
        {
            get { return m_FadeMode; }
            set { m_FadeMode = value; }
        }

        /// <summary>
        /// Sets the travel Mode for new particles
        /// </summary>
        public ParticleTravelMode ParticleTravelMode
        {
            get { return m_Mode; }
            set { m_Mode = value; }
        }
        #endregion

        /// <summary>
        /// Overloaded NewParticle Method, gives vortex particles initial velocity and sets
        /// ShouldTravel = true.
        /// </summary>
        /// <param name="newParticle"></param>
        protected override void NewParticle(XNACS1Particle newParticle)
        {
            newParticle.TravelMode = m_Mode;
            newParticle.DisplacementAmplitude = m_DispAmp;
            newParticle.DisplacementFrequency = m_DispFreq;

            Vector2 velocity;

            if (m_TravelDirection)
            {
                float theta = XNACS1Base.RandomFloat(0, (float)(2 * Math.PI));
                newParticle.CenterX += m_Radius * (float)Math.Cos(theta);
                newParticle.CenterY += m_Radius * (float)Math.Sin(theta);
                velocity = new Vector2(newParticle.CenterX - Center.X, newParticle.CenterY - Center.Y);
                velocity.Normalize();
                velocity *= -m_Speed;
            }
            else
            {
                newParticle.Center = Center;
                velocity = new Vector2(XNACS1Base.RandomFloat(-1, 1), XNACS1Base.RandomFloat(-1, 1));
                velocity.Normalize();
                velocity *= m_Speed;
            }
            newParticle.Velocity = velocity;
            newParticle.ShouldTravel = true;
            Color temp = newParticle.TextureTintColor;
            temp.A = 0;
            newParticle.TextureTintColor = temp;
        }

        /// <summary>
        /// Overloaded UpdateParticle method, makes particles fade out or in according to what
        /// percentage of the default life they have remaining.
        /// </summary>
        /// <param name="particle"></param>
        protected override void UpdateParticle(XNACS1Particle particle)
        {
            Color temp = particle.TextureTintColor;
            switch (m_FadeMode)
            {
                case VortexFadeMode.FadeOut:
                    temp.A = (byte)(255 * (((float)particle.Life) / (InitialLife)));
                    particle.TextureTintColor = temp;
                    break;
                case VortexFadeMode.FadeIn:
                    if (particle.Life > InitialLife / 2)
                    {
                        temp.A = (byte)(255 * (((float)(InitialLife - particle.Life)) / (InitialLife / 2)));
                        particle.TextureTintColor = temp;
                    }
                    break;
                case VortexFadeMode.FadeInOut:
                    if (particle.Life > 3 * InitialLife / 4)
                        temp.A = (byte)(255 * (((float)(InitialLife - particle.Life)) / (InitialLife / 4)));
                    else
                        temp.A = (byte)(255 * (((float)(particle.Life)) / (3 * InitialLife / 4)));
                    particle.TextureTintColor = temp;
                    break;
            }
        }
    }
}
